@@ -1,6 +1,6 @@
 ;;; helm-mode.el --- Enable helm completion everywhere. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2015 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2016 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -37,6 +37,8 @@
     (trace-function-foreground . helm-completing-read-symbols)
     (trace-function-background . helm-completing-read-symbols)
     (find-tag . helm-completing-read-with-cands-in-buffer)
+    (org-capture . helm-org-completing-read-tags)
+    (org-set-tags . helm-org-completing-read-tags)
     (ffap-alternate-file . nil)
     (tmm-menubar . nil)
     (find-file . nil)
@@ -364,7 +366,7 @@ Keys description:
 - EXEC-WHEN-ONLY-ONE: Bound `helm-execute-action-at-once-if-one'
   to non--nil. (possibles values are t or nil).
 
-- VOLATILE: Use volatile attribute \(enabled by default\).
+- VOLATILE: Use volatile attribute.
 
 - SORT: A predicate to give to `sort' e.g `string-lessp'
   Use this only on small data as it is ineficient.
@@ -400,7 +402,7 @@ that use `helm-comp-read' See `helm-M-x' for example."
                       . (lambda (candidate)
                           (if ,marked-candidates
                               (helm-marked-candidates)
-                            (identity candidate)))))))
+                              (identity candidate)))))))
     ;; Assume completion have been already required,
     ;; so always use 'confirm.
     (when (eq must-match 'confirm-after-completion)
@@ -479,7 +481,8 @@ that use `helm-comp-read' See `helm-M-x' for example."
                   :fuzzy-match fuzzy
                   :mode-line mode-line
                   :help-message help-message
-                  :action action-fn))
+                  :action action-fn
+                  :volatile volatile))
            (src-1 (helm-build-in-buffer-source name
                     :data get-candidates
                     :filtered-candidate-transformer fc-transformer
@@ -492,10 +495,7 @@ that use `helm-comp-read' See `helm-M-x' for example."
                     :action action-fn))
            (src-list (list src-hist
                            (if candidates-in-buffer
-                               src-1
-                             (if volatile
-                                 (append src '((volatile)))
-                               src))))
+                               src-1 src)))
            (helm-execute-action-at-once-if-one exec-when-only-one)
            (helm-quit-if-no-candidate quit-when-no-cand)
            result)
@@ -822,6 +822,11 @@ Keys description:
   ;; so always use 'confirm.
   (when (eq must-match 'confirm-after-completion)
     (setq must-match 'confirm))
+  (mapc (lambda (hook)
+          (add-hook 'helm-after-update-hook hook))
+        '(helm-ff-move-to-first-real-candidate
+          helm-ff-update-when-only-one-matched
+          helm-ff-auto-expand-to-home-or-root))
   (let* ((action-fn `(("Sole action (Identity)"
                        . (lambda (candidate)
                            (if ,marked-candidates
@@ -890,6 +895,7 @@ Keys description:
              :persistent-action persistent-action
              :persistent-help persistent-help
              :volatile t
+             :cleanup 'helm-find-files-cleanup
              :nomark nomark
              :action action-fn)))
          ;; Helm result.
@@ -1118,8 +1124,9 @@ Can be used as value for `completion-in-region-function'."
                                 (t (concat input init-space-suffix)))
                           :buffer buf-name
                           :fc-transformer (append (list 'helm-cr-default-transformer)
-                                                  (list (lambda (candidates _source)
-                                                          (sort candidates 'helm-generic-sort-fn))))
+                                                  (unless helm-completion-in-region-fuzzy-match
+                                                    (list (lambda (candidates _source)
+                                                            (sort candidates 'helm-generic-sort-fn)))))
                           :exec-when-only-one t
                           :quit-when-no-cand
                           (lambda ()
